
#include "Action.h"
#include "Configuration.h"

/**
 * Aktueller Stand
 *
 *   $RCSfile: Action.cc,v $
 *   $Revision: 1.13 $
 *   $Date: 1998/08/14 13:19:51 $
 *   $Author: myritz $ 
 *
 */

Action::Action()
  : theActionId( UNDEF_ACTION ),
    theSimStep( UNDEF_INT )
{
}

Action::Action( int actionId, int simStep, double firstParameter, double secondParameter ) 
  : theActionId( actionId ),
    theSimStep( simStep ),
    theActionCount( UNDEF_INT ),
    theFirstParameter( firstParameter ),
    theSecondParameter( secondParameter )
{
  if( actionId < MIN_DEFAULT_ACTION || actionId > MAX_DEFAULT_ACTION ) {
    throwException( ActionException( "Wrong actionId: " + actionId ) );
  }
}

Action::Action( int actionId, int simStep, int viewWidth, int viewQuality ) 
  : theActionId( actionId ), 
    theSimStep( simStep ),
    theActionCount( UNDEF_INT ),
    theViewWidth( viewWidth ), 
    theViewQuality( viewQuality ) 
{
  if( actionId != CHANGE_VIEW_ACTION ) {
    throwException( ActionException( "Wrong actionId: " + actionId ) );
  }
}

Action::Action( int actionId, int simStep, const String& message ) 
  : theActionId( actionId ), 
    theSimStep( simStep ),
    theActionCount( UNDEF_INT ),
    theMessage( message )
{
  if( actionId != SAY_ACTION ) {
    throwException( ActionException( "Wrong actionId: " + actionId ) );
  }
}

Action::~Action() 
{
}

bool Action::isValid() const {
  switch( theActionId ) {
  case MOVE_ACTION:
    if ( theFirstParameter + DEFAULT_COMPARISON_ACCURACY < -60.0 ||
        theFirstParameter - DEFAULT_COMPARISON_ACCURACY > 60.0 ||
        theSecondParameter + DEFAULT_COMPARISON_ACCURACY < -37.0 ||
        theSecondParameter - DEFAULT_COMPARISON_ACCURACY > 37.0 ) {
      return NO;
    }
    break;
  case KICK_ACTION:    
    if( theFirstParameter + DEFAULT_COMPARISON_ACCURACY < -30.0 ||
	theFirstParameter - DEFAULT_COMPARISON_ACCURACY > 100.0 ||
	theSecondParameter + DEFAULT_COMPARISON_ACCURACY < -180.0 ||
	theSecondParameter - DEFAULT_COMPARISON_ACCURACY > 180.0 ) {
      return NO;
    }
    break;
  case DASH_ACTION:    
    if( theFirstParameter + DEFAULT_COMPARISON_ACCURACY < -30.0 ||
	theFirstParameter - DEFAULT_COMPARISON_ACCURACY > 100.0 ) {
      return NO;
    }
    break;
  case TURN_ACTION:
    if( theFirstParameter + DEFAULT_COMPARISON_ACCURACY < -180.0 ||
	theFirstParameter - DEFAULT_COMPARISON_ACCURACY > 180.0 ) {
      return NO;
    }
    break;
  case CATCH_ACTION:
    if( theFirstParameter + DEFAULT_COMPARISON_ACCURACY < -180.0 ||
	theFirstParameter - DEFAULT_COMPARISON_ACCURACY > 180.0 ) {
      return NO;
    }    
    break;
  case SENSE_BODY_ACTION:
  case EMPTY_ACTION:
    break;
  case SAY_ACTION:
    if( theMessage.length() + 1 >= globalConf->sayMessageSize ) {
      return NO;
    }
    break;
  case CHANGE_VIEW_ACTION:
    switch( theViewWidth) {
    case VIEW_WIDTH_NARROW:
    case VIEW_WIDTH_NORMAL:
    case VIEW_WIDTH_WIDE:
      break;
    default:
      return NO;
    }
    switch( theViewQuality) {
    case VIEW_QUALITY_LOW:
    case VIEW_QUALITY_HIGH:
      break;
    default:
      return NO;
    }
    break;
  default:
    return NO;
  }
  return YES;
}
	
bool Action::isMainAction() const {
  switch( theActionId ) {
  case SENSE_BODY_ACTION:
  case SAY_ACTION:
  case CHANGE_VIEW_ACTION:
    return NO;
    break;
  default:
    return YES;
  }
}
	
String Action::asCommandString() const {
  String viewWidth;
  String viewQuality;

  switch( theActionId ) {
  case MOVE_ACTION:
    return( "(move " + String(theFirstParameter) + " " +
	    String(-theSecondParameter) + ")" );
    break;
  case KICK_ACTION:
    return( "(kick " + String(theFirstParameter) + " " +
	    String(-degFromRadiant(theSecondParameter)) + ")" );
    break;
  case DASH_ACTION:
    return( "(dash " + String(theFirstParameter) + ")" );
    break;
  case TURN_ACTION:
    return( "(turn " + String(-degFromRadiant(theFirstParameter)) + ")" );
    break;
  case CATCH_ACTION:
    return( "(catch " + String(-degFromRadiant(theFirstParameter)) + ")" );
    break;
  case SENSE_BODY_ACTION:
    return( "(sense_body)" );
    break;
  case EMPTY_ACTION:
    return String();
    break;
  case SAY_ACTION:
    return( "(say " + theMessage + ")" );
    break;
  case CHANGE_VIEW_ACTION:
    switch( theViewWidth ) {
    case VIEW_WIDTH_NARROW:
      viewWidth = "narrow";
      break;
    case VIEW_WIDTH_NORMAL:
      viewWidth = "normal";
      break;
    case VIEW_WIDTH_WIDE:
      viewWidth = "wide";
      break;
    }
    switch( theViewQuality ) {
    case VIEW_QUALITY_LOW:
      viewQuality = "low";
      break;
    case VIEW_QUALITY_HIGH:
      viewQuality = "high";
      break;
    }
    return( "(change_view " + viewWidth + " " + viewQuality + ")" ); 
    break;
  default:
    throwException( 
        ActionException("Invalid Action with id " + String( theActionId ) ) );
    return String();
  }
}

StorableObject* Action::copy() const {
  return new Action( *this );
}

String Action::toString() const 
{
  String output = String( "( Action for simStep " + String( simStep() ) );
  if ( id() == EMPTY_ACTION ) {
    output = output + " none )";
  } 
  else {
    output = output + String( " message: " ) + asCommandString() + " )";
  }
  return output; 
}



