#ifndef _FORWARD_PASS_OPTION_H_
#define _FORWARD_PASS_OPTION_H_

#include "KickOption.h"
#include "Field.h"

const int FORWARD_PASS_OPTION_ID = 705;


/**
  Diese Klasse steht fuer den Wunsch, den Ball in die Vorzugsrichtung zu
  schiessen. 
  
  Es wird eine Kick-Richtung ausgewaehlt, die so nahe wie moeglich an der
  Vorzugsrichtung liegt, bei der aber ein Spieler des eigenen Teams den Ball vor
  einem gegnerischen Spieler erreichen kann. Diese Option hat eine geringere
  maximale utility als die \Ref{DirectPassOption}, da ihre Umsetzung den
  Mitspieler dazu zwingt zu laufen, um den Ball zu erreichen.

  {\small {\bf Quelltext}

  [\URL[Header]{src/ForwardPassOption.h} | [\URL[Implementierung]{src/ForwardPassOption.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: ForwardPassOption.h,v $
  *   $Revision: 1.2 $
  *   $Date: 1998/08/14 13:20:34 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class ForwardPassOption : public KickOption
{
public:
  /** Erzeugt eine ForwardPassOption. */
  ForwardPassOption();

  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. Diese Methode muss fuer JEDE Unterklasse neu definiert 
      werden. */
  virtual StorableObject* copy() const;
  
protected:
  /** Der Grundnutzen des Wunsches. 
      @return Ein Wert zwischen globalConf->\Ref{maxForwardPassUtility} 
      und MIN_UTILITY. */
  int baseUtility() const;
};

#endif



