
/*
 * Aktueller Stand
 *
 *   $RCSfile: KickOption.cc,v $
 *   $Revision: 1.16 $
 *   $Date: 1998/08/14 13:20:45 $
 *   $Author: myritz $ 
 *
 */
 
#include "KickOption.h"
#include "Constraints.h"
#include "WorldModel.h"
#include "Skills.h"

KickOption::KickOption() : Option()
{
  theId = UNDEF_OPTION_ID;
}  


int KickOption::baseUtility() const 
{
  int bestUtility;
  const ModelledBall& ball = globalWorldModel->ball();
  const ModelledPlayer& player = globalWorldModel->player();  
  setIsOptionPossible( NO );
      
  if ( globalWorldModel->gameState() != WorldModel::BEFORE_KICK_OFF &&
      globalWorldModel->gameState() != WorldModel::OFFSIDE && 
      ball.posReliability().get() < Reliability::BAD && 
      player.posReliability().get() < Reliability::BAD ) {
    if ( globalWorldModel->gameState() == WorldModel::FREE_KICK &&
         globalWorldModel->initiative() == OUR_TEAM_ID && 
         globalWorldModel->simStep() < 
         globalWorldModel->simStepOfLastRefereeDecision() + 20 ) {
      bestUtility = MIN_UTILITY;
    } else {
      double currentdist = player.distTo( ball.pos() );
      INFO( Log::ACTIVE_OPTIONS, 11, "currentdist: " +
          String( currentdist ) + "\n" );     

      if ( player.distTo( ball.pos() ) < globalConf->kickRange ) { 
        // Wir koennen den Ball kicken. Die Subklassen sollten hier einen
        // differenzierteren Wert liefern
        bestUtility = MAX_UTILITY;
        setIsOptionPossible( YES );
        INFO( Log::ACTIVE_OPTIONS, 9, "Ball in kickRange\n" );     
      }
      else {
        INFO( Log::ACTIVE_OPTIONS, 9, "Ball not in kickRange\n" );     
        bestUtility = MIN_UTILITY;
      }
    }
  }
  else {
    INFO( Log::ACTIVE_OPTIONS, 9, "No sufficient reliability\n" );     
    bestUtility = NEUTRAL_UTILITY;
  }
  
  // das muessen die Subklassen ueberschreiben
  toTarget = Vector(); 
  
  return bestUtility;
}

int KickOption::constraintCost( const Constraint *constraint ) const {
  int cost;
  switch( constraint->id() ) {
    case CONSERVE_STAMINA_CONSTRAINT_ID:
    case AVOID_OFFSIDE_CONSTRAINT_ID:
    case UNDEF_CONSTRAINT_ID:
      cost = 0;
      break;
    default:
      cost = MAX_COST;
  }
  return cost;
}

int KickOption::cancellationCost() const 
{
  return NEUTRAL_COST;
}

bool KickOption::isPossible() const 
{
  expectedUtility();
  return ( toTarget.isValid() && isOptionPossible() );
}


Plan KickOption::generatePlan() const 
{
  INFO( Log::ACTIVE_OPTIONS, 3,  "ENTER KickOption::generatePlan\n" );
  Plan plan;
  if ( isPossible() == NO ) {
    WARNING( Log::ACTIVE_OPTIONS, "Generating Plan for impossible Option.\n" );
  INFO( Log::ACTIVE_OPTIONS, 3,  "LEAVE KickOption::generatePlan\n" );
    return plan;
  }
  double direction;
  double desiredPower;

  INFO( Log::ACTIVE_OPTIONS, 10, 
    "Kick vector: " + toTarget.toString() + "\n" );
  desiredPower = toTarget.length() / globalConf->kickPowerRate;
  direction = toTarget.angle();
  INFO( Log::ACTIVE_OPTIONS, 10, 
    "Desired power " + String( desiredPower ) + "\n" );

  globalSkills->resetSituation();
  plan.addPlan( globalSkills->setViewWidth( VIEW_WIDTH_NORMAL ) );
  plan.addPlan( globalSkills->kickBall( desiredPower , direction ) );
  INFO( Log::ACTIVE_OPTIONS, 3,  "LEAVE KickOption::generatePlan\n" );
  return plan;
}



 



