
/*
 * Aktueller Stand
 *
 *   $RCSfile: PassiveOption.cc,v $
 *   $Revision: 1.28 $
 *   $Date: 1998/08/14 13:21:04 $
 *   $Author: myritz $ 
 *
 */
 
#include "PassiveOption.h"
#include "Constraints.h"

PassiveOption::PassiveOption() : Option()
{
  theId = UNDEF_OPTION_ID;
}  


int PassiveOption::baseUtility() const 
{
  int utility;
  setIsPlanCompleted( NO );
  setIsTargetComputed( NO );
  // den Plan nie l"anger verfolgen, wenn nicht anders angegeben
  setForceContinuationTime( NO_CONTINUATION );
  // alle Gr\"unde bzgl. des Abbrechens der Option aktivieren
  setCancelReasons();

  const ModelledBall& ball = globalWorldModel->ball();
  const ModelledPlayer& player = globalWorldModel->player();
  
  if( player.posReliability().get() < Reliability::RANDOM ) {

    if( ( ball.posReliability().get() >= Reliability::RANDOM ) ||
        ( player.distTo( ball.pos() ) > globalConf->kickRange ) ) {   
      utility = MAX_UTILITY;
    }
    else {
      utility = MIN_UTILITY;
    }
  }
  else {
    utility = NEUTRAL_UTILITY;
  }
  return utility;
}

int PassiveOption::constraintCost( const Constraint *constraint ) const 
{
  int cost;
  // double dist;
  switch( constraint->id() ) {
    case CONSERVE_STAMINA_CONSTRAINT_ID:
      if( globalWorldModel->gameState() == WorldModel::BEFORE_KICK_OFF ||
          globalWorldModel->gameState() == WorldModel::GOAL ) {
        cost = NEUTRAL_COST;
      }
      else {
        cost = MAX_COST;
      }
      // dist = globalWorldModel->player().distTo( target() );
      // cost = (int) (dist / globalConf->distPerStaminaPoint);
      break;
    case AVOID_OFFSIDE_CONSTRAINT_ID:
      // zusaetzliche Wege werden erstmal nicht beruecksichtigt
      cost = NEUTRAL_COST;
      break;
    case UNDEF_CONSTRAINT_ID:
      cost = NEUTRAL_COST;
      break;
    default:
      cost = MAX_COST;
  }
  INFO( Log::PASSIVE_OPTIONS, 7, "Constraint cost: " + String( cost ) +"\n" );
  return cost;
}

bool PassiveOption::forceContinuation() const
{
  bool isToContinue;

  if( isCancelReasonActive( CANCEL_REASON_REFEREE_DECISION ) &&
      globalWorldModel->decisionReason() == 
      WorldModel::DECISION_REASON_REFEREE_DECISION ) {
    isToContinue = NO;
  }
  else if( isCancelReasonActive( CANCEL_REASON_NEW_BALL_INFO ) &&
           globalWorldModel->ball().posReliability().get() <
           Reliability::VERY_GOOD && globalWorldModel->decisionReason() == 
           WorldModel::DECISION_REASON_SEE_INFO ) {
    isToContinue = NO;
  }
  else if( isCancelReasonActive( CANCEL_REASON_CONT_TIME_EXPIRED ) && 
           forceContinuationTime() < globalWorldModel->simStep() ) {
    isToContinue = NO;
  }
  else if( isCancelReasonActive( CANCEL_REASON_PLAN_COMPLETED ) &&
           isPlanCompleted() ) {
    isToContinue = NO;
  }
  else {
    isToContinue = YES;
  }
  INFO( Log::PASSIVE_OPTIONS, 5, "forceContinuation answer: " + 
        String( (int) isToContinue ) + "\n" );
  return isToContinue;
}

int PassiveOption::cancellationCost() const 
{
  return NEUTRAL_COST;
}

bool PassiveOption::isPossible() const 
{
  return isOptionPossible();
}

Plan PassiveOption::movementTo( 
    const Vector& target,
    const double targetRadius, 
    const double staminaLimit,
    const int specialBehaviors ) const
{
  switch ( globalWorldModel->gameState() ) {
    case WorldModel::BEFORE_KICK_OFF:
    case WorldModel::GOAL:
      return globalSkills->beamToPosition( target );
      break;
    default:
      return globalSkills->gotoRegion( target, targetRadius, staminaLimit, 
          specialBehaviors );
      break;
  }
}

Plan PassiveOption::isSomethingToSay() const
{
  if( globalConf->useCommunication && 
      globalWorldModel->simStep() - globalWorldModel->simStepOfLastSay() >
      globalConf->hearCapacityMax ) {
    INFO( Log::PASSIVE_OPTIONS, 3, 
	  "ENTER PassiveOption::isSomethingToSay()\n" );

    const Situation& situation = globalWorldModel->situation();
    const ModelledPlayer& player = globalWorldModel->player();
    ModelledBall ball = globalWorldModel->ball();
    
    // Herausfinden ob Spieler am Ball mein Weltmodell nuetzt !
    if( player.distTo( ball.pos() ) <= 
	      min( globalConf->audioCutOffDistance,
	          globalConf->distToPlayerForSay ) ) {
      int stepsOfOurNearestPlayer = 
	        situation.stepsOfNearestPlayer(OUR_TEAM_ID);
      if( stepsOfOurNearestPlayer <= 3 && 
	        situation.stepsOfPlayerWithId( player.id() ) > 3 ) {
	      ball.simulateSteps( stepsOfOurNearestPlayer );
	      if( player.isInFieldOfView( ball ) ) {
	        double myFaceDirection = player.faceDir();
	        double himPreferredKickDirection = 
	            globalWorldModel->preferredKickDirection( ball.pos() ).angle();
	        if( fabs( normalizeAngle( myFaceDirection - 
				      himPreferredKickDirection ) ) < 
	            radFromDegree( globalConf->sayAngleDifference ) ) {
	    
	          INFO( Log::PASSIVE_OPTIONS, 3, 
		        "LEAVE PassiveOption::isSomethingToSay()\n" );
	          return globalSkills->sayWorldModel();
	        }
	      }
      }
    }
  }
  INFO( Log::PASSIVE_OPTIONS, 3, 
      "LEAVE PassiveOption::isSomethingToSay()\n" );
  return Plan();
}

Vector PassiveOption::closestNonOffsideTarget( const Vector& aTarget ) const
{
  double x;
  x = min( globalWorldModel->situation().offsideLineX() - 
           globalConf->gotoBasePositionAccuracy, aTarget.x());
  INFO( Log::PASSIVE_OPTIONS, 8, "Offside adaption: x = " +
        String( x ) + " (instead of " + String( aTarget.x() ) + ")\n" );
  return Vector( x, aTarget.y() );
}

bool PassiveOption::isDangerousPos( const Vector& aPos ) const
{
  return ( ( globalWorldModel->ourGoal().x() + 
             globalConf->dangerousRangeFromGoal) > aPos.x() ) ? YES : NO;
}

bool PassiveOption::isDefenseSituation() const
{
  if ( ( globalWorldModel->role() == DEFENDER_ROLE || 
         globalWorldModel->role() == LEARNING_DEFENDER_ROLE ||
         globalWorldModel->role() == MOVE_FORWARD_DEFENDER_ROLE ||
	       globalWorldModel->role() == MIDFIELD_ROLE || 
	       globalWorldModel->role() == LEARNING_MIDFIELD_ROLE ||
	       globalWorldModel->role() == MOVE_FORWARD_MIDFIELD_ROLE ) &&
       //       ( globalWorldModel->gameState() == WorldModel::KICK_IN ||
       //  globalWorldModel->gameState() == WorldModel::FREE_KICK ||
       //  globalWorldModel->gameState() == WorldModel::CORNER_KICK ) &&
       ( isDangerousPos( globalWorldModel->ball().pos() ) ) ) {
    return YES;
  }
  else {
    // Hier evt. debug-ausgaben, warum nicht
    return NO;
  }
}


String PassiveOption::toString() const  
{
  String output;
  output = String( " utility: " ) + String( theExpectedUtility ) + 
    String( " target: " );
  if( isTargetComputed() ) {
    output = output + target().toString();
  }
  else {
    output = output + "(not fixed)";
  }
  output = output + 
    String( " contTime: " ) + String( forceContinuationTime() ) +
    String( " cancelReasons: " ) + String( cancelReasons() ) +
    String( " isPlanCompleted: " ) + String( isPlanCompleted() ) +
    String( " isOptionPossible: " ) + String( isPossible() );

  return output;
}

 



