#ifndef _VIEWPOINT_H_
#define _VIEWPOINT_H_

#include <stream.h>
#include "Common.h"
#include "Vector.h"
#include "StorableObject.h"

/**
  Ein Blickpunkt. Ein Blickpunkt ist eine absolute Position auf dem Spielfeld
  mit einem zusaetzlichen absoluten Winkel, der Blickrichtung.

  Hauptaufgabe der Klasse Field.h ist es, aus relativen Entfernungsangaben von
  Linien/Flaggen/Toren den Blickpunkt auszurechnen. 

  {\small {\bf Quelltext}

  [\URL[Header]{src/ViewPoint.h} | [\URL[Implementierung]{src/ViewPoint.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: ViewPoint.h,v $
  *   $Revision: 1.9 $
  *   $Date: 1998/08/14 13:21:31 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class ViewPoint : public StorableObject
{
public:
  /** Default-Konstruktor. Werden keine Initialisierungswerte angegeben,
    so werden Position und Richtung auf UNDEF-Werte gesetzt. */
  ViewPoint( const Vector& aPos = Vector(), double aFaceDir = UNDEF_DOUBLE );

  /** Liefert die absolute Position des Blickpunktes. */
  const Vector& pos() const;
  
  /** Liefert die Blickrichtung (absolut und im Bogenmass). */
  double faceDir() const;
  
  /** Setzt die Position. */
  void setPos(const Vector& newPos);
  
  /** Setzt die Blickrichtung. Der Winkel wird im Bogenmass interpretiert. */
  void setFaceDir(double newFaceDir);

  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;
  
  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
    darauf zurueck. */
  virtual StorableObject* copy() const;

private:
  Vector thePos;
  double theFaceDir;
};

inline const Vector& ViewPoint::pos() const
{
  return thePos;
}

inline double ViewPoint::faceDir() const
{
  return theFaceDir;
}

inline void ViewPoint::setPos(const Vector& newPos)
{
  thePos = newPos;
}

inline void ViewPoint::setFaceDir(double newFaceDir)
{
  theFaceDir = newFaceDir;
}  
  
#endif

