
#include <stdio.h>

#include "BodyInfos.h"
#include "Exception.h"
#include "Log.h"

BodyInfos::BodyInfos( const String& information ) {
  setAllFromString(information);
}

BodyInfos::BodyInfos() {
  theTime = UNDEF_TIME;
}

void BodyInfos::setAllFromString( const String& information ) {
  if( !(information.beginsWith( "(sense_body") ) ) {
    throwException( ( Exception("BodyInfos were incorrect used") ) );
  }
  static char viewQuality[10];
  static char viewWidth[10];

  sscanf( information.cString(),
	  "(sense_body %ld (view_mode %s %s (stamina %lf %lf) (speed %lf) "
	  "(kick %d) (dash %d) (turn %d) (say %d))",
	  &theTime, viewQuality, viewWidth, &theStamina, &theEffort,
	  &theAmountOfSpeed, &theKicks, &theDashs, &theTurns, &theSays
          );

  switch( viewQuality[0] ) {
  case 'l':  // low
    theViewQuality = VIEW_QUALITY_LOW;
    break;
  case 'h':  // high
    theViewQuality = VIEW_QUALITY_HIGH;
    break;
  default:
    theViewQuality = VIEW_UNDEF;
    WARNING( Log::SENSORS, "Unknown viewQuality received" );
  }
  switch( viewWidth[1] ) { // Strings schliessen mit ')'
  case 'a': // narrow
    theViewWidth = VIEW_WIDTH_NARROW;
    break;
  case 'o': // normal
    theViewWidth = VIEW_WIDTH_NORMAL;
    break;
  case 'i': // wide
    theViewWidth = VIEW_WIDTH_WIDE;
    break;
  default:
    theViewWidth = VIEW_UNDEF;
    WARNING( Log::SENSORS, "Unknown viewWidth received" );
  }
}

StorableObject* BodyInfos::copy() const {
  return new BodyInfos( *this );
}


String BodyInfos::toString() const 
{
  String output = String( "( BodyInfos time: " ) +  String( time() ) + 
    String( ", viewQuality: " ) +  String( viewQuality() ) + 
    String( ", viewWidth: " ) +  String( viewWidth() ) +
    String( ", stamina: " ) +  String( stamina() ) +
    String( ", effort: " ) +  String( effort() ) +
    String( ", amountOfSpeed: " ) +  String( amountOfSpeed() ) +
    String( ", kicks: " ) +  String(  kicks() ) +
    String( ", dashs: " ) +  String( dashs() ) +
    String( ", turns: " ) +  String( turns() ) +
    String( ", says: " ) +  String( says() ) +  String( " )" );
  return output; 
}



