#ifndef _BODY_INFOS_H_
#define _BODY_INFOS_H_

#include "Common.h"
#include "StorableObject.h"
#include "String.h"

/**
  Informationen ueber den Zustand des eigenen Koerpers. Dieses Objekt 
  enthaelt alle Daten die einer sense_body Nachricht des SoccerServers
  zu entnehmen sind. Diese sind die Sichtqualitaet, der Sichtwinkel, 
  der Wert des Stamina, der Wert des Efforts, der Betrag der Geschwindigkeit,
  und die Anzahl der Kicks, Dashs, Turns und Says. Zusaetzlich 
  enthaelt dieses Objekt noch die Zeit zu der gerufen wurde.

  {\small {\bf Quelltext}

  [\URL[Header]{src/BodyInfos.h} | [\URL[Implementierung]{src/BodyInfos.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: BodyInfos.h,v $
  *   $Revision: 1.11 $
  *   $Date: 1998/08/14 13:19:58 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class BodyInfos : public StorableObject
{
public:
  /** 
    Default-Konstruktor. Initialisierung mit einem String. */
  BodyInfos( const String& information );

  /** Konstrukter fuer Template-Klassen */
  BodyInfos();
  
  /** Setzt alle Daten, die man dem String entnehmen kann. */
  void setAllFromString(const String& information);

  /** Die Zeit zu der die Informationen ueber den eigenen Koerper gesendet
    wurden. */
  long time() const;
  
  int viewQuality() const;
  
  int viewWidth() const;
  
  double stamina() const;
  
  double effort() const;
  
  double amountOfSpeed() const;
  
  int kicks() const;
  
  int dashs() const;
  
  int turns() const;
  
  int says() const;

  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. */
  virtual StorableObject* copy() const;

private:  
  /** Zeit zu der die Informationen ueber den eigenen Koerper gesendet wurde. */
  long theTime;

  int theViewQuality;
  int theViewWidth;
  double theStamina;
  double theEffort;
  double theAmountOfSpeed;
  int theKicks;
  int theDashs;
  int theTurns;
  int theSays;
};

inline long BodyInfos::time() const {
  return theTime;
}

inline int BodyInfos::viewQuality() const {
  return theViewQuality;
}

inline int BodyInfos::viewWidth() const {
  return theViewWidth;
}
  
inline double BodyInfos::stamina() const {
  return theStamina;
}

inline double BodyInfos::effort() const {
  return theEffort;
}
  
inline double BodyInfos::amountOfSpeed() const {
  return theAmountOfSpeed;
}

inline int BodyInfos::kicks() const {
  return theKicks;
}
  
inline int BodyInfos::dashs() const {
  return theDashs;
}

inline int BodyInfos::turns() const {
  return theTurns;
}
  
inline int BodyInfos::says() const {
  return theSays;
}

#endif







