#ifndef _DIRECT_PASS_OPTION_H_
#define _DIRECT_PASS_OPTION_H_

#include "KickOption.h"
#include "Field.h"

const int DIRECT_PASS_OPTION_ID = 706;


/**
  Diese Klasse steht fuer den Wunsch, den Ball direkt zu einem Mitspieler zu
  spielen. 
  
  Es wird ein Spieler ausgewaehlt, der so nahe wie moeglich an der
  Vorzugsrichtung liegt und den Ball vor einem gegnerischen Spieler erreichen
  kann. Diese Option hat eine hoehere maximale Nuetzlichkeit als die
  \Ref{ForwardPassOption}, da ihre Umsetzung den Mitspieler nicht dazu zwingt
  zu laufen, um den Ball zu erreichen.
  
  Diese Option wird von der Praktikumsgruppe 'attack' implementiert.

  {\small {\bf Quelltext}

  [\URL[Header]{src/DirectPassOption.h} | [\URL[Implementierung]{src/DirectPassOption.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: DirectPassOption.h,v $
  *   $Revision: 1.2 $
  *   $Date: 1998/08/14 13:20:17 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class DirectPassOption : public KickOption
{
public:
  /** Erzeugt eine DirectPassOption. */
  DirectPassOption();

  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. Diese Methode muss fuer JEDE Unterklasse neu definiert 
      werden. */
  virtual StorableObject* copy() const;
  
protected:  
  /** Der Grundnutzen des Wunsches. Dieser Teil der Nutzenberechnung ist nur 
      vom Weltmodell abhaengig.
      @return Im Allgemeinen ein Wert zwischen MAX_UTILITY und MIN_UTILITY. */
  int baseUtility() const;
};

#endif



