#ifndef _MESSAGE_H_
#define _MESSAGE_H_

#include "Common.h"
#include "StorableObject.h"
#include "String.h"

/**
  Eine gehoerte Nachricht. Eine Nachricht besteht aus der eigentlichen
  Mitteilung (einem String) und einem Absender. Der Absender ist entweder eine
  Spieler- oder die Schiedsrichter-ID oder eine Richtung. Zusaetzlich 
  enthaelt dieses Objekt noch die Zeit zu der gerufen wurde.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Message.h} | [\URL[Implementierung]{src/Message.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Message.h,v $
  *   $Revision: 1.12 $
  *   $Date: 1998/08/14 13:20:49 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class Message : public StorableObject
{
public:

  /** 
    Default-Konstruktor. Initialisierung mit einem String. */
  Message( const String& information );

  /** Konstrukter fuer Template-Klassen */
  Message();  

  /** Setzt alle Daten, die man dem String entnehmen kann. */
  void setAllFromString(const String& information);
  
  /** Die Zeit zu der die Nachricht gesendet wurde. */
  long time() const;

  /* Liefert die Nachricht zurueck. */
  const String& message() const;

  /** Liefert Infos ueber den Rufer. Moegliche Rueckgabewerte sind:  SELF_ID,
    OUR_TEAM_ID (eigenes Team, aber nicht der Agent selbst),
    THEIR_TEAM_ID, REFEREE_ID, UNDEF_TEAM_ID. */
  int senderInfo() const;
  
  /** Beschreibung als String fuer die Ausgabe auf einem Terminal.
      Es werden ID und Position ausgegeben. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. */
  virtual StorableObject* copy() const;
  
private:
  /** Untersucht den Messagestring und setzt die Instanzvariablen.
    Wurde dabei entdeckt, dass ein Gegner eine eigene Nachricht wiederholt hat,
    wird eine Warnung ins Log geschrieben. */
  void parseMessageString();
  
  
  /** Zeit zu der die Nachricht gesendet wurde. */
  long theTime;

  /** Der Klartext der Nachricht. */
  String theMessage;
  
  /** Absender-Info. Das, was sender() zurueckliefert. Der Wert wird nur intern
    von parseMessageString() gesetzt. */
  int theSenderInfo; 
};

inline long Message::time() const {
  return theTime;
}

inline const String& Message::message() const {
  return theMessage;
}

inline int Message::senderInfo() const {
  return theSenderInfo;
}

#endif
