#ifndef _PLAN_H_
#define _PLAN_H_

#include "Action.h"
#include "Array.h"
#include "StorableObject.h"
#include "String.h"


/**
  Ausfuehrbarer Plan. 
  Ein Plan enthaelt eine Reihe von Basisaktionen.
  Plaene werden im Planer/Intention erzeugt und von der Effektorenkomponente 
  ausgefuehrt.

  Plan ist nicht zur Vererbung geeignet.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Plan.h} | [\URL[Implementierung]{src/Plan.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Plan.h,v $
  *   $Revision: 1.16 $
  *   $Date: 1998/08/14 13:21:06 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class Plan : public StorableObject
{
public:
  /** Erzeugt einen neuen Plan. */
  Plan();
	
  /** Erzeugt einen neuen Plan mit initialer Laenge von length. */
  Plan(int length);

  /** Loescht den Plan. */
  ~Plan();
	
  /** Haengt eine Aktion an den Plan an. */
  void addAction( const Action& newAction );
  
  /** Haengt einen Plan an den Plan an. */
  void addPlan( const Plan& newPlan );

  /** Liefert die Aktion die an der angegebene Stelle (index) steht.
      @param index Index der Aktion im Array. */
  Action& actionAt(int index) const;
  
  
  /** Liefert das Array mit den Aktionen. */
  const Array<Action> actions() const;
  
  /** Gibt die Zahl der Aktionen im Plan zurueck. */
  int length() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. */
  virtual StorableObject* copy() const;
  
  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;
  
private:
  Array<Action> theActions;
};

inline void Plan::addAction(const Action& newAction) {
  theActions.addObject(newAction);
}

inline Action& Plan::actionAt(int index) const {
  return theActions.objectAt(index);
}

inline const Array<Action> Plan::actions() const {
  return theActions;
}

inline int Plan::length() const {
  return theActions.count();
}

#endif
