
/*
 * Aktueller Stand
 *
 *   $RCSfile: Vector.cc,v $
 *   $Revision: 1.16 $
 *   $Date: 1998/08/14 13:21:28 $
 *   $Author: myritz $ 
 *
 */


#include "Vector.h"
#include "Common.h"
#include "String.h"
#include <math.h>

Vector::Vector(double value1, double value2, 
               CoordinateSystemEnum aCoordinateSystem = CARTESIAN, 
               AngleSystemEnum anAngleSystem = RADIAN ) :
  theWasInitializedWithNegativeDist( NO )
{
  if ( value1 == UNDEF_DOUBLE || value2 == UNDEF_DOUBLE) {
    theX = UNDEF_DOUBLE;
    theY = UNDEF_DOUBLE;
  }
  else {
    if ( aCoordinateSystem == CARTESIAN ) {
      theX = value1;
      theY = value2;
    }
    else if ( aCoordinateSystem == POLAR ) {
      if( value1 < 0 ) {
        theWasInitializedWithNegativeDist = YES;
      }
      if ( anAngleSystem == RADIAN ) {
	      theX = value1 * cos( value2 ); 
	      theY = value1 * sin( value2 );
      }
      else if ( anAngleSystem == DEGREE ) {
	      theX = value1 * cos( radFromDegree( value2 ) );
	      theY = value1 * sin( radFromDegree( value2 ) );
      }
      else{
        throwException( VectorException( "unknown AngleSystem" ) );
      }
    }
    else{
      throwException( VectorException( "unknown CoordinateSystem" ) );
    }
  } 
}


Vector::Vector() :
  theX( UNDEF_DOUBLE ),
  theY( UNDEF_DOUBLE ),
  theWasInitializedWithNegativeDist( NO )
{
}


double Vector::length() const
{  
  if ( theX == UNDEF_DOUBLE || theX == UNDEF_DOUBLE ) {
    return UNDEF_DOUBLE;
  }
  return sqrt( pow( theX, 2 ) + pow( theY, 2 ) );
}
  

double Vector::angle( AngleSystemEnum anAngleSystem = RADIAN ) const
{
  double myAngle;
  
  if ( theX == UNDEF_DOUBLE || theY == UNDEF_DOUBLE ) {
    return UNDEF_DOUBLE;
  }
  if ( theX == 0 ) {
    if ( theY < 0 ) {
      myAngle = - M_PI/2;
    }
    else if ( theY > 0 ) {
      myAngle = M_PI/2;
    }
    else {
	    myAngle = 0;
    }
  }
  else {
    myAngle = atan( theY / theX );
    if ( theX < 0 ) {
      if ( theY >= 0 ) {
        myAngle += M_PI;
      }
      else {
        myAngle -= M_PI;
      }
    }
  }
  if ( anAngleSystem == RADIAN ) {
    return myAngle;
  }
  else if ( anAngleSystem == DEGREE ) {
    return ( degFromRadiant( myAngle ) );
  }
  else {
    throwException( VectorException( "unknown AngleSystem" ) );
  }
}


void Vector::setPolar( double aLength, double anAngle,
                       AngleSystemEnum anAngleSystem = RADIAN )
{

  if ( aLength == UNDEF_DOUBLE || anAngle == UNDEF_DOUBLE ) {
    theX = UNDEF_DOUBLE;
    theY = UNDEF_DOUBLE;
  }
  else {
    if( aLength < 0 ) {
      theWasInitializedWithNegativeDist = YES;
    }
    if ( anAngleSystem == RADIAN ){
    }
    else if ( anAngleSystem == DEGREE ) {
      anAngle = radFromDegree( anAngle );
    }
    else {
      throwException( VectorException( "unknown AngleSystem" ) );
    }
    theX = cos( anAngle ) * aLength;
    theY = sin( anAngle ) * aLength;
  }
}


void Vector::set( double value1, double value2, 
                  CoordinateSystemEnum aCoordinateSystem = CARTESIAN,
                  AngleSystemEnum anAngleSystem = RADIAN  )
{
  if ( aCoordinateSystem == CARTESIAN ) {
    setCartesian( value1, value2 );
  }
  else if ( aCoordinateSystem == POLAR ) {
    setPolar( value1, value2, anAngleSystem );
  }
  else {
    throwException( VectorException( "unknown CoordinateSystem" ) );
  }
}


void Vector::normalize() {
  double myLength = length();
  if ( myLength != 0 ) {
    theX /= myLength;
    theY /= myLength;
  }
  else {
    theX = UNDEF_DOUBLE;
    theY = UNDEF_DOUBLE;
  }
}


StorableObject* Vector::copy() const {
  return new Vector( *this );
}


String Vector::toString() const 
{
  String output;
  if ( !isValid() ) {
    output = "( ungueltiger Vektor )";
  }
  else {
    output = String( "( kart: ( " ) + String( theX ) + String(", ") +
      String( theY ) + String( "), polar: ( " ) + String( length() ) +
      String( ", " ) + String( angle(DEGREE) ) + String( ") )" );
  }
  return output;
}



